#!/usr/bin/env python
# ex:ts=2

# (c) 2004-2006 Tilman Linneweh
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the above copyright notice is
# retained.
#
# Getting started
# ###############
#
# * Type rssfetch.py -h to read the help message
# * Define a class rssitems in your stylesheet
#
# $Id: rssfetch.py,v 1.11 2006/03/08 18:44:21 arved Exp $

import feedparser
import re
from getopt import *
import sys
import cgi

version = "0.3"
##
# Customize titles:
# foo is replaced with bar

mytitles = [
	# ['foo', 'bar'],
	['Wibble', 'lauri - Wibble'],
	['Autoblogiographie', 'Hendrik Scholz - Autoblogiographie'],
]

## RSSFetch class

class RSSFetch:

	def htmlEncode(self,s):
		"""Return the HTML encoded version of the given string.

		This is useful to display a plain ASCII text string on a web page.

		"""
		codes = [
			#['&', '&amp;'],
			['<', '&lt;'],
			['>', '&gt;'],
			['"', '&quot;'],
		#	['\n', '<br>']
		]
		for code in codes:
			s = s.replace(code[0], code[1])
		return s

	def fixuptitle(self,s):
		s = re.sub("\(mailto:.*","",s)
		for title in mytitles:
			s.replace(title[0], title[1])
		return s

	def processFeed(self,url,numberofitems,fw,adddate,debug):
		tags = re.compile('<.*?>', re.M)
		try:
			d = feedparser.parse(url)
		except:
			print "Could not parse feed: "+ url
			return -1
		if debug:
			print "Encoding: " + d.encoding
			print d.feed
		if d.feed.has_key('title'):
			if d.feed.has_key('author') and d.feed.author != "":
				htmltitle = cgi.escape((d.feed.author + " - " + d.feed.title).encode(d.encoding))
			else:
				htmltitle = cgi.escape(d.feed.title.encode(d.encoding))
			htmltitle = self.fixuptitle(htmltitle)
			htmltitle = tags.sub('', htmltitle)
		else:
			if debug: print "Title not found"
			return -1
		if debug:
			print "Link: " + cgi.escape(d.feed.link)
			print "Title: " + htmltitle
		print >> fw, "<a href=\"" + cgi.escape(d.feed.link) + "\">"
		print >> fw, htmltitle
		print >> fw, "</a>"
		feeditems = len(d['entries'])
		if feeditems <= numberofitems:
			numberofitems = feeditems
		if debug: print "Number of items: "+str(numberofitems)
		print >> fw, "<div class=\"rssitems\">"
		for a in range(0,numberofitems):
			e = d.entries[a]
			if e.has_key('title'):
				htmlentry = tags.sub('',e.title.encode(d.encoding))
				htmlentry = self.htmlEncode(htmlentry)
			else:
				if debug: print "Entry name not found!"
				adddate = 1
				htmlentry = ""
			if adddate:
				if e.has_key('issued'):
					htmlentry = e.issued + " " + htmlentry
				elif e.has_key('created'):
					htmlentry = e.created + " " + htmlentry
				elif e.has_key('modified'):
					htmlentry = e.modified + " " + htmlentry
				else:
					if debug: print e
					continue
			htmlentry = cgi.escape(htmlentry)
			if debug:
				print "Link: " + cgi.escape(e.link)
				print "Entry: " + htmlentry
			print >> fw, "<a href=\"" + cgi.escape(e.link) + "\">"
			print >> fw, htmlentry
			print >> fw, "</a><br />"
		print >> fw, "</div>"
		return 0

## main

def main():
	adddate= 0
	numberofitems = 3
	infile = "blogroll.en"
	outfile = "blogroll.en.inc"
	uacomment = "http://arved.priv.at/rssfetch/ "
	debug= 0

	opts,args=getopt(sys.argv[1:],'c:df:gho:u:vF:V')

	for o,a in opts:
		if o=="-c": numberofitems= int(a)
		if o=="-d": adddate=1
		if o=="-f": infile=a
		if o=="-g": debug=1
		if o=="-h":
			print "usage: $prog [-cdefhvV]"
			print "\t -c Number	Number of topics displayed"
			print "\t -d		Add date to topic"
			print "\t -F URL		feed"
			print "\t -f filename	configurationfile"
	 		print "\t -u comment	Useragent comment"
			print "\t -o filename	outputfile"
			print "\t -g		debugoutput"
			print "\t -h		summary of command line options"
			print "\t -v		verbose output"
			print "\t -V		print the version and exit"
			return
		if o=="-o": outfile=a
		if o=="-u": uacomment=a
		if o=="-v": debug=1
		if o=="-F":
			fw=open(outfile,"w")
			rf = RSSFetch()
			rf.processFeed(a,numberofitems,fw,adddate,debug)
			return
		if o=="-V":
			print "py-rssfetch " + version
			print "(c) 2004-2006 Tilman Linneweh"
			return

	if debug:
		print "Running with debugging output"
	feedparser.USER_AGENT = "py-rssfetch/" + version + " - " + uacomment
	fr=open(infile,"r")
	fw=open(outfile,"w")
	for a in fr.readlines():
		if a[0] != "#":
			regex=re.compile("((((https?|ftp)://)|(mailto|file|news):)[^ \"\'<>;:,]+)",re.I)
			rxp = regex.search(a)
			if not rxp == None:
				a = rxp.group()
			if debug:
				print "Processing: " + a
			rf = RSSFetch()
			rf.processFeed(a,numberofitems,fw,adddate,debug)
	fr.close()
	fw.close()

if __name__ == "__main__":
    main()

